@@ -19,6 +19,10 @@
  * MA 02110-1301 USA
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
@@ -32,8 +36,6 @@
 #include "context-private.h"
 #include "timer.h"
 
-#include <libdivecomputer/custom_io.h>
-
 struct dc_context_t {
 	dc_loglevel_t loglevel;
 	dc_logfunc_t logfunc;
@@ -42,8 +44,6 @@ struct dc_context_t {
 	char msg[8192 + 32];
 	dc_timer_t *timer;
 #endif
-	dc_custom_io_t *custom_io;
-	dc_user_device_t *user_device;
 };
 
 #ifdef ENABLE_LOGGING
@@ -178,8 +178,6 @@ dc_context_new (dc_context_t **out)
 	dc_timer_new (&context->timer);
 #endif
 
-	context->custom_io = NULL;
-
 	*out = context;
 
 	return DC_STATUS_SUCCESS;
@@ -191,30 +189,14 @@ dc_context_free (dc_context_t *context)
 	if (context == NULL)
 		return DC_STATUS_SUCCESS;
 
+#ifdef ENABLE_LOGGING
 	dc_timer_free (context->timer);
+#endif
 	free (context);
 
 	return DC_STATUS_SUCCESS;
 }
 
-dc_status_t
-dc_context_set_custom_io (dc_context_t *context, dc_custom_io_t *custom_io, dc_user_device_t *user_device)
-{
-	if (context == NULL)
-		return DC_STATUS_INVALIDARGS;
-
-	context->custom_io = custom_io;
-	custom_io->user_device = user_device;
-
-	return DC_STATUS_SUCCESS;
-}
-
-dc_custom_io_t*
-_dc_context_custom_io (dc_context_t *context)
-{
-	return context->custom_io;
-}
-
 dc_status_t
 dc_context_set_loglevel (dc_context_t *context, dc_loglevel_t loglevel)
 {
@@ -337,3 +319,35 @@ dc_context_hexdump (dc_context_t *context, dc_loglevel_t loglevel, const char *f
 
 	return DC_STATUS_SUCCESS;
 }
+
+unsigned int
+dc_context_get_transports (dc_context_t *context)
+{
+	UNUSED(context);
+
+	return DC_TRANSPORT_SERIAL
+#if defined(HAVE_LIBUSB)
+		| DC_TRANSPORT_USB
+#endif
+#if defined(HAVE_HIDAPI)
+		| DC_TRANSPORT_USBHID
+#elif defined(HAVE_LIBUSB) && !defined(__APPLE__)
+		| DC_TRANSPORT_USBHID
+#endif
+#ifdef _WIN32
+#ifdef HAVE_AF_IRDA_H
+		| DC_TRANSPORT_IRDA
+#endif
+#ifdef HAVE_WS2BTH_H
+		| DC_TRANSPORT_BLUETOOTH
+#endif
+#else /* _WIN32 */
+#ifdef HAVE_LINUX_IRDA_H
+		| DC_TRANSPORT_IRDA
+#endif
+#ifdef HAVE_BLUEZ
+		| DC_TRANSPORT_BLUETOOTH
+#endif
+#endif /* _WIN32 */
+	;
+}
